﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class SphereStateNames {
	public static readonly SphereStateNames NullStateID = new SphereStateNames(0); // Use this ID to represent a non-existing State in your system
	/// <summary>We can't move.</summary>
	public static readonly SphereStateNames Stopped = new SphereStateNames(1);
	/// <summary>We moving like an old rpg.</summary>
	public static readonly SphereStateNames FourDir = new SphereStateNames(2);
	/// <summary>We moving like a racecar.</summary>
	public static readonly SphereStateNames Car = new SphereStateNames(3);

	public int Value { get; protected set; }

	protected SphereStateNames(int internalValue) {
		this.Value = internalValue;
	}
}

public class SphereController : MonoBehaviour {

	public GameObject Sphere;

	private SphereState _currentState;
	private List<SphereState> _listOfStates = new List<SphereState>();

	private Material _material;

	void Awake() {
		_material = GetComponent<Renderer>().material;

		AddStates();
	}

	// Use this for initialization
	void Start () {
		GoToState(SphereStateNames.Stopped);
	}
	
	// Update is called once per frame
	void Update () {
		if (Input.GetKeyDown(KeyCode.Space)) {
			if (_currentState.StateName == SphereStateNames.Stopped) {
				GoToState(SphereStateNames.FourDir);
			} else if (_currentState.StateName == SphereStateNames.FourDir) {
				GoToState(SphereStateNames.Car);
			} else {
				GoToState(SphereStateNames.Stopped);
			}

			return;
		}

		float xAxis, yAxis;

		xAxis = Input.GetAxis("Horizontal");
		yAxis = Input.GetAxis("Vertical");

		if (xAxis <= -0.15) {
			xAxis = -1;
		} else if (xAxis >= 0.15) {
			xAxis = 1;
		} else {
			xAxis = 0f;
		}

		if (yAxis <= -0.15) {
			yAxis = -1;
		} else if (yAxis >= 0.15) {
			yAxis = 1;
		} else {
			yAxis = 0f;
		}

		_currentState.ReceiveInput(xAxis, yAxis);
	}

	void AddStates() {
		_listOfStates.Add(new SphereStateStopped(this));
		_listOfStates.Add(new SphereStateFourDir(this));
		_listOfStates.Add(new SphereStateCar(this));
	}

	public void GoToState(SphereStateNames newStateName) {
		foreach (SphereState nState in _listOfStates) {
			if (nState.StateName == newStateName) {
				if (_currentState != null) {
					_currentState.OnEnd(newStateName);
				}

				_currentState = nState;
				_currentState.OnBegin(_currentState != null ? _currentState.StateName : SphereStateNames.NullStateID);

				return;
			}
		}

		Debug.LogWarning("SphereController::GoToState a state with the given name was not found.");
	}

	public void ChangeSphereColor(Color value) {
		_material.SetColor("_Color", value);
	}

	public void TranslateSphere(float xAxis, float yAxis, Space relativeTo = Space.Self) {
		Sphere.transform.Translate(xAxis, yAxis, 0f, relativeTo);
	}

	public void SetRotationSphere(Quaternion facing) {
		Sphere.transform.rotation = facing;
	}
}
